
library(s2dv)
library(easyVerification)

ROCSS <- function(exp, obs, ref = NULL, n_categories, time_dim = 'year', member_dim = 'member', ncores = NULL){
  
  ## Reordering dimensions
  exp <- s2dv::Reorder(data = exp, order = c(names(dim(exp))[-(which(names(dim(exp)) %in% c(time_dim,member_dim)))], time_dim, member_dim))
  obs <- s2dv::Reorder(data = obs, order = c(names(dim(exp))[-(which(names(dim(exp)) %in% c(time_dim,member_dim)))], time_dim))
  
  ## ROCS for the forecast
  rocs_exp <- easyVerification::veriApply(verifun = 'EnsRoca', fcst = exp, obs = obs, 
                                          tdim = which(names(dim(exp))==time_dim), 
                                          ensdim = which(names(dim(exp))==member_dim), 
                                          prob = (1:(n_categories-1))/n_categories, 
                                          na.rm = FALSE, ncpus = ncores)
  ## ROCSS
  rocss <- list()
  if (is.null(ref)){ 
    ## Climatology as reference forecast
    for (cat in paste0('cat',1:n_categories)){
      rocss[[cat]] <- 2 * rocs_exp[[cat]] - 1
    }
  } else {
    ## Reordering dimensions
    ref <- s2dv::Reorder(data = ref, order = c(names(dim(ref))[-(which(names(dim(ref)) %in% c(time_dim,member_dim)))], time_dim, member_dim))
    ## ROCS for the reference forecast
    rocs_ref <- easyVerification::veriApply(verifun = 'EnsRoca', fcst = ref, obs = obs, 
                                            tdim = which(names(dim(ref))==time_dim), 
                                            ensdim = which(names(dim(ref))==member_dim), 
                                            prob = (1:(n_categories-1))/n_categories, 
                                            na.rm = FALSE, ncpus = ncores)
    for (cat in paste0('cat',1:n_categories)){
      rocss[[cat]] <- (rocs_exp[[cat]] - rocs_ref[[cat]]) / (1 - rocs_ref[[cat]])
    }
  }

  return(rocss)
}
