library(startR)
library(multiApply)
library(s2dv)
library(CSTools)
library(ClimProjDiags)
library(easyVerification)
library(zoo)
library(SPEI)
library(TLMoments)
library(lmom)
library(lmomco)
source('startR_calls.R')
source('P_PET_Accum.R')
source('SPEI_from_Accum.R')
source('CorrEno.R')
source('RMSSS_edited.R')
source('ROCSS.R')
source('get_probs.R')
source('plot_deterministic_forecast.R')
source('plot_probabilistic_forecast.R')
.WeightedMean <- ClimProjDiags:::.WeightedMean
options(bitmapType = 'cairo') # to avoid error when plotting in Nord3

config <- list(
  ## Paths
  outputs_path = '/esarchive/scratch/ptrascas/',
  metrics_path = '/esarchive/scratch/ptrascas/',
  figures_metrics_path = '/esarchive/scratch/ptrascas/',
  figures_forecast_path = '/esarchive/scratch/ptrascas/',
  figures_regional_path = '/esarchive/scratch/ptrascas/',
  figures_path = '/esarchive/scratch/ptrascas/',
  ## Data
  observations = c('ERA5land'),
  models = c('CanESM5','EC-Earth3','HadGEM3-GC31-MM'),
  members = list('CanESM5' = paste0('r',1:20,'i1p2f1'),
                 'EC-Earth3' = paste0('r',1:10,'i4p1f1'),
                 'HadGEM3-GC31-MM' = paste0('r',1:10,'i1p1f2')),
  ## Period
  years = 1963:2023,
  sdates = 1962:2023,
  fperiods = list(1, 1:5),
  ## Region
  lon_min = 0,
  lon_max = 50,
  lat_min = -10,
  lat_max = 10,
  regions = list('Guatemana-Coban' = list(lon = -90.38, lat = 15.46), 
                 'Colombia-ElMeta' = list(lon = -73.63, lat = 4.15),
                 'Colombia-ValleMagdalena' = list(lon = -73.85, lat = 7.06),
                 'IberianPeninsula' = list(lon_min = -10.8, lon_max = 4.2, lat_min = 36, lat_max = 44),
                 'Colombia' = list(lon_min = -80.35, lon_max = -66.06, lat_min = -4, lat_max = 11.7)),
  ## Computation
  indices_months = list('3' = c(3,6,9,12), '12' = 12),
  pet_method = 'hargreaves',
  spei_method = 'parametric',
  spei_distribution = 'log-Logistic', 
  spei_fit = 'ub-pwm',
  grid = 'r180x90',
  interpolation_method = 'con',
  calibration_methods = c('False','bias','evmos','mse_min','crps_min','rpc-based'),
  n_categories = 3,
  alpha = 0.05,
  na.rm = FALSE,
  ncores = 16,
  retrieve = TRUE,
  job_wallclock = '01:00:00',
  max_jobs = 10
)

## Creating the folders for the results (in case they don't exit yet)
if (!dir.exists(config$outputs_path)){dir.create(config$outputs_path, recursive = TRUE)}
if (!dir.exists(config$metrics_path)){dir.create(config$metrics_path, recursive = TRUE)}
if (!dir.exists(config$figures_metrics_path)){dir.create(config$figures_metrics_path, recursive = TRUE)}
if (!dir.exists(config$figures_forecast_path)){dir.create(config$figures_forecast_path, recursive = TRUE)}
if (!dir.exists(config$figures_regional_path)){dir.create(config$figures_regional_path, recursive = TRUE)}
if (!dir.exists(config$figures_path)){dir.create(config$figures_path, recursive = TRUE)}

