
library(multiApply)
library(easyVerification)

get_probs <- function(data, type, indices_for_quartiles, time_dim, member_dim, n_categories){
  
  if (type == 'exp'){
    target_dims_quantiles = c(time_dim, member_dim)
    target_dims_probs = list(member_dim, 'bin')
  } else if (type == 'obs'){
    target_dims_quantiles = time_dim
    target_dims_probs = list(NULL,'bin')
  } else {stop("Parameter 'type' must be 'exp' or 'obs'")}
  
  quantiles <- multiApply::Apply(data = ClimProjDiags::Subset(x = data, along = time_dim, indices = indices_for_quartiles), 
                                 target_dims = target_dims_quantiles, 
                                 fun = .quantile_aux, n_categories = n_categories, 
                                 output_dims = 'bin', ncores = NULL)$output1
  
  probs <- multiApply::Apply(data = list(data = data, quantiles = quantiles),
                             target_dims = target_dims_probs,
                             fun = .c2p, 
                             output_dims = 'bin', ncores = NULL)$output1
  
  return(list('probs' = probs, 'quantiles' = quantiles))
}

.quantile_aux = function(data, n_categories){
  data = as.vector(data)
  q = quantile(x = data, probs = 1:(n_categories - 1) / n_categories, type = 8, na.rm = TRUE)
  return(q)
}

.c2p <- function(data, quantiles) {
  if (anyNA(data)){
    output <- array(data = NA, dim = length(quantiles) + 1)
  } else {
    output <- colMeans(easyVerification::convert2prob(as.vector(data), threshold = as.vector(quantiles)))
  }
  return(output)
}

