Sys.setenv(LANG = "en")
Sys.setlocale("LC_TIME","en_GB")
library(startR)
library(s2dverification)
library(abind)
library(CSTools)
library(zeallot)
library(s2dv)
library(easyVerification)
library(ClimProjDiags)
library(ggplot2)
library(openxlsx)


dates <- format(seq(as.Date("20240812", "%Y%m%d"), as.Date("20240826", "%Y%m%d"), 7), "%Y%m%d")
var_obs <- "tas"

## LOCATIONS ##

locations <- list(
#	dcoop_jardinete = c(lat = 39.25, lon = 356.93),
#	dcoop_rebollo = c(lat = 39.25, lon = 356.97),
#	dcoop_fherminia = c(lat = 39.35, lon = 356.93),
#	dcoop_migueleta = c(lat = 39.38, lon = 356.92),
#	dcoop_manzaneque = c(lat = 39.43, lon = 356.88),
#	dcoop_pozo = c(lat = 39.5, lon = 3.13),
#	dcoop_s57zr = c(lat = 39.55, lon = 3.12),
#	dcoop_mjose = c(lat = 39.6, lon = 3.12),

#	mcodax_cambados_orosal = c(lat = 41.9, lon = 351.18),
#	mcodax_cambados_pontevedra = c(lat = 42.45, lon = 351.3), 
#	mcodax_cambados_vgarousa = c(lat = 42.48, lon = 351.3),
#	mcodax_meano_vgarousa = c(lat = 42.50, lon = 351.3) ,
#	mcodax_meano_pontevedra = c(lat = 42.50, lon = 351.3), 
#	mcodax_ribadumia_vgarousa = c(lat = 42.51, lon = 351.3), 
#	mcodax_sanxenxo_vgarousa = c(lat = 42.53, lon = 351.3), 
#	mcodax_vgarousa_vgarousa = c(lat = 42.53, lon = 351.35), 
#	mcodax_vnarousa_vgarousa = c(lat = 42.53, lon = 351.3), 
#	mcodax_vlardevos = c(lat = 42.56, lon = 352.63),

	matarromera_sanroman = c(lat = 41.64, lon = 355.68),
	matarromera_lasmarcas = c(lat = 41.36, lon = 355.24),

	vriotajo = c(lat = 39.87, lon = 355.87)
)

location_names <- names(locations)
patho <- "/esarchive/recon/ecmwf/era5land/weekly_mean/"

results_array <- array(NA, dim = c(length(locations), length(dates), 1),
			dimnames = list(
				loc = location_names,
				dates = dates,
				variable = var_obs
			))

for (j in seq_along(dates)) {
	date <- dates[j]
	path_obs <- paste0(patho, "tas_f1h/tas_", date, ".nc")
	cat("Processing date:", date, "\n")  # Print the current date
	obs_date <- Start(dat = path_obs, var = var_obs, lat = "all", lon = "all",
			  time = "all", retrieve = T)

	obs_date <- obs_date - 273.15
	
	for (i in seq_along(locations)) {
		loc_name <- location_names[i]
		loc <- locations[[i]]
		lat_val <- values(loc[[1]])
		lon_val <- values(loc[[2]])
		cat (" Processing location: ", loc_name,"\n")

		obs_loc <- Subset(
				  obs_date,
				  along = c("lat", "lon"),
				  indices = list(
               	 		                lat = lat_val, 
						lon = lon_val), 
            			  drop = T)
		results_array[i,j,1] <- obs_loc
	}
}

obs_date <- obs_date - 273.15

# Convert the array into a data frame with proper headers
results_df <- as.data.frame(results_array)

# Add loc as the header for the first column
results_df <- cbind(loc = rownames(results_df), results_df)

# Assign proper column names
colnames(results_df) <- c("loc", dates)

# Print the final data frame
print(results_df)

# Transpose the data frame
transposed_df <- as.data.frame(t(results_df[-1]))  # Exclude the first column ("loc") for transposition
colnames(transposed_df) <- results_df$loc          # Use location names as column headers

# Add dates as the first column
transposed_df <- cbind(dates = rownames(transposed_df), transposed_df)

# Save the transposed table as a CSV file
write.csv(transposed_df, file = "tas.csv", row.names = FALSE)



