Sys.setenv(LANG = "en")
Sys.setlocale("LC_TIME","en_GB")
library(startR)
library(s2dverification)
library(abind)
library(CSTools)
library(zeallot)
library(s2dv)
library(easyVerification)
library(ClimProjDiags)
library(ggplot2)

region <- 'cat'
# There are some issues picking the location, numbers must be "exact"
Granollers <- c(lat = 41.61, lon = 2.3)

## LOAD OBSERVATIONS OF THE TARGET WEEK ##

var_obs <- "tasmax"
target <- c('20240722')
patho <- "/esarchive/recon/ecmwf/era5land/weekly_mean/"
path_obs <- paste0(patho,"tasmax_f24h","/tasmax_20240722.nc")
obs <- Start(dat = path_obs,
                   var = "tasmax",
                   lat = values(Granollers[[1]]),
                   lon = values(Granollers[[2]]),
                   time = 'all',
                   return_vars = list(lat = NULL, lon = NULL),
                   retrieve = T)
as.vector(attributes(obs)$Variables$common$latitude)
as.vector(attributes(obs)$Variables$common$longitude)
attributes(obs)$Variables$common$time
dim(attributes(obs)$Variables$common$time)
obs <- obs - 273.15

## LOAD OBSERVATIONS TO ESTIMATE PERCENTILE OF OBS AND TRANSLATE IT INTO HINDCAST ##
var_obs <- "tasmax"
target_week <- c('0722')
patho <- "/esarchive/recon/ecmwf/era5land/weekly_mean/"
obs_years <- seq(from = 1999, to = 2016, by = 1)
dates <- paste0(obs_years,target_week)
path_obs <- sapply(dates, function(date) {
  paste0(patho, "tasmax_f24h/tasmax_", date, ".nc")
})
#path_obs <- paste0(patho,"tasmax_f24h/tasmax_$dates$.nc")
obs_clim <- Start(dat = path_obs,
                   var = var_obs,
		   #sdates = dates,
                   lat = values(Granollers[[1]]),
                   lon = values(Granollers[[2]]),
                   time = 'all',
		   retrieve = T)
as.vector(attributes(obs)$Variables$common$latitude)
as.vector(attributes(obs)$Variables$common$longitude)
attributes(obs)$Variables$common$time
dim(attributes(obs)$Variables$common$time)
obs_clim <- obs_clim - 273.15

obs_per <- sum(obs_clim <= obs[1]) /length(obs_clim) * 100



## LOAD FORECAST ##
## Don't forget to chose I leadtime per startdate ##
sdates <- c("20240627","20240718")

var <- "t02max"
path <- "/esarchive/exp/VITIGEOSS/subseasonal/"
path_fcst <- paste0(path, var, "/$sdates$/", var, "_", region, "_$sdates$.nc")
fcst <- Start(dat = path_fcst,
                  var = var,
                  sdates = sdates,
                  latitude = values(Granollers[[1]]),
                  longitude = values(Granollers[[2]]),
                  ensemble = 'all',
                  time = 'all',
                  retrieve = T)
                  as.vector(attributes(fcst)$Variables$common$latitude)
                  as.vector(attributes(fcst)$Variables$common$longitude)

fcst_1 <- fcst[1,1,1,1,1,,4]
fcst_2 <- fcst[1,1,2,1,1,,1]

obs_new <- quantile(fcst_2, probs = obs_per / 100)
obs_new <- obs_new[[1]]

# PROBABILITIES
path_probs <- paste0(path, var, "/$sdates$/", var, "_", region, "_prob_$sdates$.nc")
probs <- Start(dat = path_probs,
                   var = c('prob_bn', 'prob_n', 'prob_an'),
                   sdates = sdates,
                   latitude = values(Granollers[[1]]),
                   longitude = values(Granollers[[2]]),
                   time = 'all',
                   return_vars = list(latitude = NULL, longitude = NULL, time = 'sdates'),
                   retrieve = T)
as.vector(attributes(probs)$Variables$common$latitude)
as.vector(attributes(probs)$Variables$common$longitude)
attributes(probs)$Variables$common$time
dim(attributes(probs)$Variables$common$time)

probs_1 <- probs[1,,1,1,1,4]

# TERCILE LIMITS
# Load tercile limits of target week

week <- format(seq(as.Date("20240104", "%Y%m%d"), as.Date("20241226", "%Y%m%d"), 7), "%Y%m%d")
weeknum <- sprintf("%02d", which(week == sdates))

path_lims <- paste0(path, var, "/$sdates$/", var, "_", region,
             "_percentiles_week$weeknum$.nc")
dim(week) <- c(weeks = length(week), sdates = 1)
lims <- Start(dat = path_lims,
                  var = c('p33', 'p66'),
                  sdates = sdates,
                  weeknum = 'all',
                  weeknum_depends = 'sdates',
                  latitude = values(Granollers[[1]]),
                  longitude = values(Granollers[[2]]),
                  time = 'all',
                  return_vars = list(latitude = NULL, longitude = NULL, time = 'sdates'),
                  retrieve = T)

 as.vector(attributes(lims)$Variables$common$latitude)
 as.vector(attributes(lims)$Variables$common$longitude)
 attributes(lims)$Variables$common$time
 dim(attributes(lims)$Variables$common$time)
 print(lims)

lims_1 <- lims[1,,1,1,1,1,4]

lims <- rbind(lims_1)

## SKILL ##

path_skill <- paste0(path, var, "/$sdates$/", var, "_", region,
                         "_skill_week$weeknum$.nc")
rpss <- Start(dat = path_skill,
                  var = c('rpss'),
                  sdates = sdates,
                  weeknum = 'all',
                  weeknum_depends = 'sdates',
                  latitude = values(Granollers[[1]]),
                  longitude = values(Granollers[[2]]),
                  time = 'all',
                  return_vars = list(latitude = NULL, longitude = NULL, time = 'sdates'),
                  retrieve = T)
as.vector(attributes(rpss)$Variables$common$latitude)
as.vector(attributes(rpss)$Variables$common$longitude)
attributes(rpss)$Variables$common$time
dim(attributes(rpss)$Variables$common$time)

rpss_1 <- rpss[1,1,1,1,1,1,4]

# plot the observations 
obs_1 <- obs_new

obs <- c(obs_1)

fcst <- data.frame(fcst1 = fcst_1)

mondays <- sapply(1:1, function(x) {format(seq(as.Date(sdates[x], "%Y%m%d") + 4, as.Date(sdates[x], "%Y%m%d") + 30, 7), "%d-%m-%Y")})
sundays <- sapply(1:1, function(x) {format(seq(as.Date(sdates[x], "%Y%m%d") + 10, as.Date(sdates[x], "%Y%m%d") + 40, 7), "%d-%m-%Y")})
rpss <- c(rpss_1)
sdates <- sapply(1:1, function(x) {format(as.Date(sdates[x], "%Y%m%d"), "%d-%m-%Y")})
subtit <- sapply(1:1, function(x) { paste0("Emitida el \n", sdates[x], "\nRPSS = ", round(rpss[x]/100,digits=2))})

figpath <- "/esarchive/scratch/ptrascas/R/sunset/I4C_oper_eval/"

targett <- format(as.Date(target, "%Y%m%d"), "%d-%m-%Y") 


PlotForecastPDF(fcst,
		tercile.limits = lims,
		obs = obs,
		var.name = "Temperatura maxima (\u00B0C)",
		fcst.names = subtit,
		title = paste0("Prediccion de temperatura maxima en Granollers \npara la semana del ",
                format(as.Date(target, "%Y%m%d"), "%d-%m-%Y")), color.set="s2s4e",
                add.ensmemb = "no", plotfile = paste0(figpath, "/eval_I4C_targetpdf_",var,"_Granollers_10_good.png"))
