
library(startR)

startR_calls <- function(variable, dataset, fyears, config){
  
  ##############################################################################################################################
  #################################################### OBSERVATIONS ############################################################
  ##############################################################################################################################
  
  if (identical(dataset,'ERA5land')){
    
    if (identical(variable,'pr')) {variable <- 'prlr'}
    aux <- list('tasmin' = 'f24h', 'tasmax' = 'f24h', 'prlr' = 'f1h')
    
    data <- startR::Start(dat = '/esarchive/recon/ecmwf/era5land/monthly_mean/$var$_$aux$/$var$_$year$$month$.nc', 
                          var = variable,  lon = values(list(config$lon_min,config$lon_max)), lat = values(list(config$lat_min,config$lat_max)),
                          year = paste0(config$years), month = 'all', aux = aux[[variable]],
                          transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = config$grid, method = config$interpolation_method), transform_vars = c('lon','lat'),
                          return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = config$ncores, retrieve = config$retrieve)
    units_tas <- 'K'
    units_pr <- 'm s-1'
    
    ##############################################################################################################################
    ################################################# DECADAL PREDICTIONS ########################################################
    ##############################################################################################################################
    
  } else if (identical(dataset,'CanESM5')){
    
    exp_list <- list()
    for (sdt in config$sdates){if (sdt <= 2019){exp_list[[as.character(sdt)]] <- 'dcppA-hindcast'} else {exp_list[[as.character(sdt)]] <- 'dcppB-forecast'}}
    times <- (1 + 12 * (fyears[1] - 1)) : (12 * fyears[length(fyears)]) ## init in January
    
    data <- startR::Start(dat = '/esarchive/exp/CMIP6/$exp$/CanESM5/DCPP/CCCma/CanESM5/$exp$/$member$/Amon/$var$/gn/v20190429/$var$_Amon_CanESM5_$exp$_s$sdate$-$member$_gn_$aux$.nc',
                          var = variable, lon = values(list(config$lon_min,config$lon_max)), lat = values(list(config$lat_min,config$lat_max)),
                          sdate = paste0(config$sdates), fmonth = indices(times), aux = 'all', member = config$members[[dataset]], exp = exp_list,
                          aux_depends = 'sdate', exp_depends = 'sdate', exp_depends = 'aux', aux_depends = 'exp',
                          transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = config$grid, method = config$interpolation_method), transform_vars = c('lon','lat'),
                          synonims = list(fmonth = c('fmonth','time')),
                          return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = config$ncores, retrieve = config$retrieve)
    units_tas <- 'K'
    units_pr <- 'kg m-2 s-1'
    
  } else if (identical(dataset,'EC-Earth3')){
    
    exp_list <- list(); 
    for (sdt in config$sdates){if (sdt <= 2020){exp_list[[as.character(sdt)]] <- 'dcppA-hindcast'} else {exp_list[[as.character(sdt)]] <- 'dcppB-forecast'}}
    times <- (3 + 12 * (fyears[1] - 1)) : (12 * fyears[length(fyears)] + 2) ## init in November
    
    data <- startR::Start(dat = '/esarchive/exp/ecearth/a3w5/original_files/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/$exp$/$member$/Amon/$var$/gr/v20210910/$var$_Amon_EC-Earth3_$exp$_s$sdate$-$member$_gr_$fyear$.nc',
                          var = variable, lon = values(list(config$lon_min,config$lon_max)), lat = values(list(config$lat_min,config$lat_max)),
                          sdate = paste0(config$sdates), fmonth = indices(times), fyear = 'all', member = config$members[[dataset]], exp = exp_list, 
                          fmonth_across = 'fyear', merge_across_dims = TRUE,
                          fyear_depends = 'sdate', exp_depends = 'sdate', exp_depends = 'fyear', fyear_depends = 'exp', 
                          transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = config$grid, method = config$interpolation_method), transform_vars = c('lon','lat'),
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), fmonth = c('fmonth','time')),
                          return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = config$ncores, retrieve = config$retrieve)
    units_tas <- 'K'
    units_pr <- 'kg m-2 s-1'
    
  } else if (identical(dataset,'HadGEM3-GC31-MM')){
    
    exp_list <- list()
    for (sdt in config$sdates){
      if (sdt <= 2018){
        exp_list[[as.character(sdt)]] <- 'dcppA-hindcast'
      } else {
        exp_list[[as.character(sdt)]] <- 'dcppB-forecast'
      }
    }
    times <- (1 + 12 * (fyears[1] - 1)) : (12 * fyears[length(fyears)]) ## init in November but files start in January
    
    if (variable %in% c('tasmin','tasmax')){
      symbol <- '_' ## inconsistencies of esarchive
    } else {symbol <- '-'}
    
    data <- startR::Start(dat = paste0('/esarchive/exp/CMIP6/$exp$/HadGEM3-GC31-MM/DCPP/MOHC/HadGEM3-GC31-MM/$exp$/$member$/Amon/$var$/gn/v20200417/$var$_Amon_HadGEM3-GC31-MM_$exp$_s$sdate$-$member$_gn_$aux$01',symbol,'$aux$12.nc'), 
                          var = variable, lon = values(list(config$lon_min,config$lon_max)), lat = values(list(config$lat_min,config$lat_max)),
                          sdate = paste0(config$sdates), fmonth = indices(times), aux = 'all', member = config$members[[dataset]], exp = exp_list,
                          aux_depends = 'sdate', aux_depends = 'exp', exp_depends = 'sdate', exp_depends = 'aux',
                          transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = config$grid, method = config$interpolation_method), transform_vars = c('lon','lat'),
                          fmonth_across = 'aux', merge_across_dims = TRUE, synonims = list(fmonth = c('fmonth','time')),
                          return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = config$ncores, retrieve = config$retrieve)
    units_tas <- 'K'
    units_pr <- 'kg m-2 s-1'
    
  } else {stop('Incorrect dataset')}
  
  lon <- attr(data,'Variables')$common[['lon']]
  lat <- attr(data,'Variables')$common[['lat']]
  stopifnot(!is.null(lat))
  stopifnot(!is.null(lon))
  stopifnot(is.null(attr(data,'NotFoundFiles')))
  
  return(list(data = data, 
              lat = lat, 
              lon = lon,
              units_tas = units_tas, 
              units_pr = units_pr))
}
